import enum


class TipoArquivo(enum.Enum):
    TXT = "T"
    DBF = "D"


class Arquivo(enum.Enum):
    ARQUIVO_DE_APLICACAO_RESGATE = "002"
    ARQUIVO_DE_COTACOES = "003"
    CADASTRO_DE_FUNDOS = "004"
    OPERACOES_EM_SER_DO_PASSIVO = "005"
    CADASTRO_DE_GESTORES = "006"
    CADASTRO_DE_GRUPOS = "007"
    CADASTRO_DE_USUARIOS = "008"
    CADASTRO_DE_ASSISTENTES = "009"
    ARQUIVO_DE_OPERACOES_EM_SER_MENSAL = "010"
    POSICAO_DE_FUNDOS_DE_PRIVATIZACAO = "011"
    ARQUIVO_DE_PERFORMANCE = "012"
    ARQUIVO_CAC_TOTAL = "013"
    ARQUIVO_ARA_PARCIAL = "014"
    ARQUIVO_ARA_TOTAL = "015"
    ARQUIVO_RAF_PARCIAL = "016"
    ARQUIVO_RES_PARCIAL = "017"
    ARQUIVO_DE_CONTROLE_DE_ARQUIVOS_GERADOS = "018"
    ARQUIVO_DE_CADASTRO_DE_COTITULARES = "019"
    OPERACOES_EM_SER_DO_PASSIVO_D1 = "021"
    OPERACOES_EM_SER_DO_PASSIVO_D2 = "022"
    ARQUIVO_DE_EXTRATO_COMPENSACAO_DE_PREJUIZO = "023"
    RECOLHIMENTO_DE_TRIBUTOS_POR_PERIODO = "024"
    POSICAO_RESULTANTE_DE_RENDIMENTOS_A_COMPENSAR = "025"
    FECHAMENTO_DE_COTAS_EM_SER_D0 = "026"
    FECHAMENTO_DE_COTAS_EM_SER_D1 = "027"
    FECHAMENTO_DE_COTAS_EM_SER_D2 = "028"
    APLICACOES_EM_ABERTO_COTAS_DE_ABERTURA = "029"
    APLICACOES_EM_ABERTO_COTAS_DE_ABERTURA_D1 = "030"
    APLICACOES_EM_ABERTO_COTAS_DE_ABERTURA_D2 = "031"
    APLICACOES_EM_ABERTO_COTAS_DE_FECHAMENTO = "032"
    APLICACOES_EM_ABERTO_COTAS_DE_FECHAMENTO_D1 = "033"
    APLICACOES_EM_ABERTO_COTAS_DE_FECHAMENTO_D2 = "034"
    ARQUIVO_DE_MOVIMENTACOES_EFETIVADAS = "035"
    ARQUIVO_DE_MOVIMENTACOES_PENDENTES = "036"
    CADASTRO_DE_COTISTAS_EXPANDIDO = "037"
    ARQUIVO_ARA_CERTIFICADO_PARCIAL = "038"
    ARQUIVO_ARA_CERTIFICADO_TOTAL = "039"
    ARQUIVO_DE_CONTAS_DE_CREDITO_INVESTIMENTO = "041"
    ARQUIVO_DE_BLOQUEIOS = "050"
    ARQUIVO_DE_CLASSIFICACAO_DE_COTISTAS = "051"
    EXTRATO_COMPENSACAO_DE_PREJUIZO_FECHAMENTO = "053"
    RECOLHIMENTO_DE_TRIBUTOS_POR_DECENDIO = "056"
    OPERACOES_EM_SER_REGERADAS = "057"
    APLICACOES_EM_ABERTO_REGERADAS = "058"
    ARQUIVO_DE_SALDOS_COTA_DE_ABERTURA_D0 = "062"
    ARQUIVO_DE_SALDOS_COTA_DE_ABERTURA_D_1 = "063"
    ARQUIVO_DE_SALDOS_COTA_DE_ABERTURA_D_2 = "064"
    ARQUIVO_DE_SALDOS_COTA_DE_FECHAMENTO_D0 = "065"
    ARQUIVO_DE_SALDOS_COTA_DE_FECHAMENTO_D_1 = "066"
    ARQUIVO_DE_SALDOS_COTA_DE_FECHAMENTO_D_2 = "067"
    ARQUIVO_DE_SALDOS_MENSAL = "068"
    PESSOAS_VINCULADAS_DO_COTISTA = "069"
    CADASTRO_DE_CONTAS_DE_CREDITO_COMPLETO_D_1 = "073"
    MOVIMENTO_DIARIO_D_1 = "074"
    OPERACOES_EMSER_DO_PASSIVO_SIMULADA = "075"
    ARQUIVO_DE_RESGATES_AGENDADOS = "076"
    RELACAO_DE_DOCUMENTOS_PENDENTES = "077"
    ARQUIVO_DE_EXTRATO_COMPENSACAO_DE_PREJUIZO_ = "078"
    EXTRATO_COMPENSACAO_DE_PREJUIZO_CPF_FECHA = "079"
    MOVIMENTO_HISTORICO_DE_PREJUIZO_CPF = "080"
    POSICAO_RESULTANTE_DE_RENDIMENTOS_A_COMPENSAR_ = "081"
    ARQUIVO_DE_OPERACOES_DE_EXTRATO_REPROCESSADO = "082"
    CADASTRO_DE_COTISTAS_INTRAG = "083"
    PES_VINCULADAS_COTISTA_INTRAG = "084"
    CADASTRO_CO_TITULARES_INTRAG = "085"
    ACOMP_GIIN_PENDENTE_FATCA = "086"
    RISK_POINT_INTRAG = "089"
    ENVIO_DE_CORRESPONDENCIA = "091"
    E_FINANCEIRA_SALDOS = "092"
    E_FINANCEIRA_MOVIMENTOS = "093"
    E_FINANCEIRA_PAGAMENTOS = "094"
    MEMORIA_DE_CALCULO_AMORTIZACAO = "095"
    ARQUIVO_DE_SALDOS_AMORTIZADOS = "096"
    ISS_INFORMACAO_MENSAL_MERCADO = "097"
    CONSULTA_DE_MOVIMENTOS_CETIP = "098"
    CONSULTA_DE_CONTAS_CETIP = "099"
    POSICAO_COTISTA_DISTRIBUIDOR = "100"
    MOVIMENTOS_SIMULADOS_D0 = "101"
    INCONSISTENCIAS_CADASTRAIS = "102"
    ARQUIVO_TCR_GERAL = "103"
